﻿const express = require('express');
const Redis = require('ioredis');
const bodyParser = require('body-parser');

const app = express();
let redis; // Declare Redis connection variable

app.use(bodyParser.json());
app.use(express.static('public')); // Static file serving

// Redis 연결 함수
const connectRedis = () => {
    redis = new Redis();
    redis.on('connect', () => console.log('Connected to Redis.'));
    redis.on('error', (err) => console.error('Redis error:', err));
};

// Initialize Redis connection
connectRedis();

// 데이터 전송
app.post('/send', async (req, res) => {
    const sendData = req.body;
    console.log('Received data to send:', sendData);
    
    try {
        const jsonText = JSON.stringify(sendData);

        // 키의 데이터 유형 확인
        const recvkey = 'CLI:1127001637919216817849';
        const type = await redis.type(recvkey);
        
        if (type !== 'none' && type !== 'list') {
            // 키가 존재하고 리스트가 아닌 경우 삭제
            await redis.del(recvkey);
        }

        await redis.rpush(recvkey, jsonText);
        console.log('Data sent to Redis:', jsonText);
        res.json({ message: 'Data sent successfully.' });
    } catch (error) {
        console.error('Failed to send data to Redis:', error);
        res.status(500).json({ error: 'Failed to send data.' });
    }
});

// 데이터 수신
app.get('/receive', async (req, res) => {
    console.log('Request received to fetch data from Redis');
    
    try {
		const sendkey = 'QCS:1127001637919216817849';
        const result = await redis.blpop(sendkey, 0); // 0 means it will block until data is available
        if (result) {
            const data = JSON.parse(result[1]); // result[1] is the value
            return res.json(data); // Return the data as a JSON response
        }
    } catch (err) {
        console.error('Error fetching data from Redis:', err);
        return res.status(500).json({ error: 'Failed to fetch data.' });
    }
    
    return res.status(404).json({ message: 'No data found.' });
});

// Redis 연결 종료
const quitRedis = async () => {
    if (redis) {
        try {
            await redis.quit(); // Close the Redis connection
            console.log('Redis connection closed.');
        } catch (error) {
            console.error('Failed to close Redis connection:', error);
        }
    }
};

// 서버 시작
const PORT = 3000;
app.listen(PORT, () => {
    console.log(`Server is running on http://localhost:${PORT}`);
});

// Gracefully shutdown the server and close the Redis connection
process.on('SIGINT', async () => {
    await quitRedis();
    process.exit(0);
});
